@@ -7,8 +7,6 @@ from .const import VERSION
 from .exception_classes import LoginError
 from .gql_handler import GQLHandler, NormalStatus, WatchOnlineStatus
 
-
-
 _LOGGER = logging.getLogger(__name__)
 
 
@@ -26,21 +24,10 @@ class PyXploraApi:
         self.maxRetries = 3
         self.retryDelay = 2
 
-        self.dtIssueToken = int(time.time()) - (self.tokenExpiresAfter * 1000)
-
-        self.contacts = []
-        self.alarms = []
-        self.chats = []
-        self.safe_zones = []
-        self.school_silent_mode = []
-        self.watch_location = []
-        self.watch_last_location = []
-        self.watch_battery = None
-        self.watch_charging = None
+        self.dtIssueToken = int(time()) - (self.tokenExpiresAfter * 1000)
 
         self.__gqlHandler = None
         self.__issueToken = None
-        # self.watch = None
 
         self.watchs = []
 
@@ -66,7 +53,7 @@ class PyXploraApi:
                         if (not self.__issueToken):
                             sleep(self.retryDelay)
                     if (self.__issueToken):
-                        self.dtIssueToken = int(time.time())
+                        self.dtIssueToken = int(time())
                 else:
                     raise Exception("Unknown error creating a new GraphQL handler instance.")
             except Exception:
@@ -83,13 +70,12 @@ class PyXploraApi:
         self.__issueToken = None
 
     def __hasTokenExpired(self) -> bool:
-        return ((int(time.time()) - self.dtIssueToken) > (self.tokenExpiresAfter * 1000))
+        return ((int(time()) - self.dtIssueToken) > (self.tokenExpiresAfter * 1000))
 
     def init(self, forceLogin=False) -> None:
         token = self.__login(forceLogin)
         if token:
             if ('user' in token):
-                # self.watch = token['user']['children'][self.watch_no]['ward']
                 if not self._childPhoneNumber:
                     self.watchs = token['user']['children']
                 else:
@@ -108,6 +94,7 @@ class PyXploraApi:
         retryCounter = 0
         dataOk = False
         contacts_raw = None
+        contacts = []
         while (not dataOk and (retryCounter < self.maxRetries + 2)):
             retryCounter += 1
             self.init()
@@ -115,8 +102,12 @@ class PyXploraApi:
                 contacts_raw = self.__gqlHandler.getContacts(watchID)
                 if 'contacts' in contacts_raw:
                     if contacts_raw['contacts'] is None:
-                        continue
+                        dataOk = True
+                        return contacts
                     if 'contacts' in contacts_raw['contacts']:
+                        if not contacts_raw['contacts']['contacts']:
+                            dataOk = True
+                            return contacts
                         for contact in contacts_raw['contacts']['contacts']:
                             try:
                                 xcoin = contact['contactUser']['xcoin']
@@ -125,7 +116,7 @@ class PyXploraApi:
                                 # None - XCoins
                                 xcoin = -1
                                 id = None
-                            self.contacts.append({
+                            contacts.append({
                                 'id': id,
                                 'guardianType': contact['guardianType'],
                                 'create': datetime.fromtimestamp(contact['create']).strftime('%Y-%m-%d %H:%M:%S'),
@@ -136,12 +127,12 @@ class PyXploraApi:
                             })
             except Exception as error:
                 _LOGGER.debug(error)
-            dataOk = self.contacts
+            dataOk = contacts
             if (not dataOk):
                 self.__logoff()
                 sleep(self.retryDelay)
         if dataOk:
-            return self.contacts
+            return contacts
         else:
             raise Exception('Xplora API call finally failed with response: ')
 
@@ -208,6 +199,7 @@ class PyXploraApi:
         retryCounter = 0
         dataOk = False
         alarms_raw = None
+        alarms = []
         while (not dataOk and (retryCounter < self.maxRetries + 2)):
             retryCounter += 1
             self.init()
@@ -216,10 +208,11 @@ class PyXploraApi:
                 if 'alarms' in alarms_raw:
                     if not alarms_raw['alarms']:
                         dataOk = True
-                        return self.alarms
+                        return alarms
                     for alarm in alarms_raw['alarms']:
-                        self.alarms.append({
+                        alarms.append({
                             'id': alarm['id'],
+                            'vendorId': alarm['vendorId'],
                             'name': alarm['name'],
                             'start': self.__helperTime(alarm['occurMin']),
                             'weekRepeat': alarm['weekRepeat'],
@@ -227,20 +220,20 @@ class PyXploraApi:
                         })
             except Exception as error:
                 _LOGGER.debug(error)
-            dataOk = self.alarms
+            dataOk = alarms
             if (not dataOk):
                 self.__logoff()
                 sleep(self.retryDelay)
         if (dataOk):
-            return self.alarms
+            return alarms
         else:
             raise Exception('Xplora API call finally failed with response: ')
-        return self.alarms
 
     def loadWatchLocation(self, withAsk=True, watchID=0) -> list:
         retryCounter = 0
         dataOk = False
         location_raw = None
+        watch_location = []
         while (not dataOk and (retryCounter < self.maxRetries + 2)):
             retryCounter += 1
             self.init()
@@ -251,7 +244,7 @@ class PyXploraApi:
                 location_raw = self.__gqlHandler.getWatchLastLocation(watchID)
                 if 'watchLastLocate' in location_raw:
                     if location_raw['watchLastLocate'] is not None:
-                        self.watch_location.append({
+                        watch_location.append({
                             'tm': datetime.fromtimestamp(location_raw['watchLastLocate']['tm']).strftime('%Y-%m-%d %H:%M:%S'),
                             'lat': location_raw['watchLastLocate']['lat'],
                             'lng': location_raw['watchLastLocate']['lng'],
@@ -263,27 +256,25 @@ class PyXploraApi:
                             'locateType': location_raw['watchLastLocate']['locateType'],
                             'isInSafeZone': location_raw['watchLastLocate']['isInSafeZone'],
                             'safeZoneLabel': location_raw['watchLastLocate']['safeZoneLabel'],
+                            'watch_battery': location_raw['watchLastLocate']['battery'],
+                            'watch_charging': location_raw['watchLastLocate']['isCharging'],
+                            'watch_last_location': location_raw['watchLastLocate'],
                         })
-                        self.watch_battery = location_raw['watchLastLocate']['battery']
-                        self.watch_charging = location_raw['watchLastLocate']['isCharging']
-                        self.watch_last_location = location_raw['watchLastLocate']
             except Exception as error:
                 _LOGGER.debug(error)
-            dataOk = self.watch_location
+            dataOk = watch_location
             if (not dataOk):
                 self.__logoff()
                 sleep(self.retryDelay)
         if (dataOk):
-            return self.watch_location
+            return watch_location
         else:
             raise Exception('Xplora API call finally failed with response: ')
 
     def getWatchBattery(self, watchID) -> int:
-        self.loadWatchLocation(watchID=watchID)
-        return self.watch_battery
+        return self.loadWatchLocation(watchID=watchID)[0]['watch_battery']
     def getWatchIsCharging(self, watchID) -> bool:
-        self.loadWatchLocation(watchID=watchID)
-        if self.watch_charging:
+        if self.loadWatchLocation(watchID=watchID)[0]['watch_charging']:
             return True
         return False
     def getWatchOnlineStatus(self, watchID) -> WatchOnlineStatus:
@@ -322,6 +313,7 @@ class PyXploraApi:
         retryCounter = 0
         dataOk = False
         chats_raw = None
+        chats = []
         while (not dataOk and (retryCounter < self.maxRetries + 2)):
             retryCounter += 1
             self.init()
@@ -331,8 +323,11 @@ class PyXploraApi:
                 chats_raw = self.__gqlHandler.chats(watchID)
                 if 'chats' in chats_raw:
                     if 'list' in chats_raw['chats']:
+                        if not chats_raw['chats']['list']:
+                            dataOk = True
+                            return chats
                         for chat in chats_raw['chats']['list']:
-                            self.chats.append({
+                            chats.append({
                                 'msgId': chat['msgId'],
                                 'type': chat['type'],
                                 # chat['sender'],
@@ -347,35 +342,31 @@ class PyXploraApi:
                             })
             except Exception as error:
                 _LOGGER.debug(error)
-            dataOk = self.chats
+            dataOk = chats
             if (not dataOk):
                 self.__logoff()
                 sleep(self.retryDelay)
         if (dataOk):
-            return self.chats
+            return chats
         else:
             raise Exception('Xplora API call finally failed with response: ')
 
 ##### Watch Location Info #####
     def getWatchLastLocation(self, withAsk: bool = False, watchID=0) -> dict:
-        self.loadWatchLocation(withAsk, watchID=watchID)
-        return self.watch_last_location
+        return self.loadWatchLocation(withAsk, watchID=watchID)[0]['watch_last_location']
     def getWatchLocate(self, watchID) -> dict:
-        self.loadWatchLocation(watchID=watchID)
-        return self.watch_location
+        return self.loadWatchLocation(watchID=watchID)[0]
     def getWatchLocateType(self, watchID) -> str:
-        self.getWatchLocate(watchID)
-        return self.watch_last_location['locateType']
+        return self.getWatchLocate(watchID)['locateType']
     def getWatchIsInSafeZone(self, watchID) -> bool:
-        self.getWatchLocate(watchID)
-        return self.watch_last_location['isInSafeZone']
+        return self.getWatchLocate(watchID)['isInSafeZone']
     def getWatchSafeZoneLabel(self, watchID) -> str:
-        self.getWatchLocate(watchID)
-        return self.watch_last_location['safeZoneLabel']
+        return self.getWatchLocate(watchID)['safeZoneLabel']
     def getSafeZones(self, watchID) -> list:
         retryCounter = 0
         dataOk = False
         safeZones_raw = None
+        safe_zones = []
         while (not dataOk and (retryCounter < self.maxRetries + 2)):
             retryCounter += 1
             self.init()
@@ -384,9 +375,11 @@ class PyXploraApi:
                 sleep(self.retryDelay)
                 safeZones_raw = self.__gqlHandler.safeZones(watchID)
                 if 'safeZones' in safeZones_raw:
+                    if not safeZones_raw['safeZones']:
+                        dataOk = True
+                        return safe_zones
                     for safeZone in safeZones_raw['safeZones']:
-                        self.safe_zones.append({
-                            # safeZone,
+                        safe_zones.append({
                             'vendorId': safeZone['vendorId'],
                             'groupName': safeZone['groupName'],
                             'name': safeZone['name'],
@@ -397,12 +390,12 @@ class PyXploraApi:
                         })
             except Exception as error:
                 _LOGGER.debug(error)
-            dataOk = self.safe_zones
+            dataOk = safe_zones
             if (not dataOk):
                 self.__logoff()
                 sleep(self.retryDelay)
         if (dataOk):
-            return self.safe_zones
+            return safe_zones
         else:
             raise Exception('Xplora API call finally failed with response: ')
     def trackWatchInterval(self, watchID) -> int:
@@ -414,31 +407,36 @@ class PyXploraApi:
     def schoolSilentMode(self, watchID) -> list:
         retryCounter = 0
         dataOk = False
-        sientTimes_raw = None
+        silentTimes_raw = None
+        school_silent_mode = []
         while (not dataOk and (retryCounter < self.maxRetries + 2)):
             retryCounter += 1
             self.init()
             try:
                 self.askWatchLocate(watchID)
                 sleep(self.retryDelay)
-                sientTimes_raw = self.__gqlHandler.silentTimes(watchID)
-                if 'silentTimes' in sientTimes_raw:
-                    for sientTime in sientTimes_raw['silentTimes']:
-                        self.school_silent_mode.append({
-                            'id': sientTime['id'],
-                            'start': self.__helperTime(sientTime['start']),
-                            'end': self.__helperTime(sientTime['end']),
-                            'weekRepeat': sientTime['weekRepeat'],
-                            'status': sientTime['status'],
+                silentTimes_raw = self.__gqlHandler.silentTimes(watchID)
+                if 'silentTimes' in silentTimes_raw:
+                    if not silentTimes_raw['silentTimes']:
+                        dataOk = True
+                        return school_silent_mode
+                    for silentTime in silentTimes_raw['silentTimes']:
+                        school_silent_mode.append({
+                            'id': silentTime['id'],
+                            'vendorId': silentTime['vendorId'],
+                            'start': self.__helperTime(silentTime['start']),
+                            'end': self.__helperTime(silentTime['end']),
+                            'weekRepeat': silentTime['weekRepeat'],
+                            'status': silentTime['status'],
                         })
             except Exception as error:
                 _LOGGER.debug(error)
-            dataOk = self.school_silent_mode
+            dataOk = school_silent_mode
             if (not dataOk):
                 self.__logoff()
                 sleep(self.retryDelay)
         if (dataOk):
-            return self.school_silent_mode
+            return school_silent_mode
         else:
             raise Exception('Xplora API call finally failed with response: ')
     def setEnableSilentTime(self, silentId, watchID) -> bool:
@@ -573,7 +571,7 @@ class PyXploraApi:
         raise Exception("no Admin")
 
 ##### - #####
-    def __helperTime(self, time) -> str:
-        h = str(int(time) / 60).split('.')
+    def __helperTime(self, t) -> str:
+        h = str(int(t) / 60).split('.')
         h2 = str(int(h[1]) * 60).zfill(2)[:2]
         return h[0].zfill(2) + ":" + str(h2).zfill(2)
